@@ -11,8 +11,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Fastimport/fastexport commands."""
 
@@ -280,14 +279,6 @@ class cmd_fast_import(Command):
         from bzrlib.plugins.fastimport.helpers import (
             open_destination_directory,
             )
-        # If no format is given and the user is running a release
-        # leading up to 2.0, select 2a for them. Otherwise, use
-        # the default format.
-        if format is None:
-            import bzrlib
-            bzr_version = bzrlib.version_info[0:2]
-            if bzr_version in [(1,17), (1,18), (2,0)]:
-                format = bzrdir.format_registry.make_bzrdir('2a')
         control = open_destination_directory(destination, format=format)
 
         # If an information file was given and the source isn't stdin,
@@ -388,6 +379,15 @@ class cmd_fast_import_filter(Command):
         User mapping is supported by both the fast-import and
         fast-import-filter commands.
 
+    :History rewriting:
+
+     By default fast-import-filter does quite aggressive history rewriting.
+     Empty commits (or commits which had all their content filtered out) will
+     be removed, and so are the references to commits not included in the stream.
+
+     Flag --dont-squash-empty-commits reverses this behavior and makes it possible to
+     use fast-import-filter on incremental streams.
+
     :Examples:
 
      Create a new project from a library (note the trailing / on the
@@ -416,16 +416,30 @@ class cmd_fast_import_filter(Command):
                     Option('user-map', type=str,
                         help="Path to file containing a map of user-ids.",
                         ),
+                    Option('dont-squash-empty-commits',
+                        help="Preserve all commits and links between them"
+                        ),
                      ]
     encoding_type = 'exact'
     def run(self, source=None, verbose=False, include_paths=None,
-        exclude_paths=None, user_map=None):
+        exclude_paths=None, user_map=None, dont_squash_empty_commits=False):
+        from bzrlib.errors import BzrCommandError
         load_fastimport()
         from fastimport.processors import filter_processor
         params = {
             'include_paths': include_paths,
             'exclude_paths': exclude_paths,
             }
+        if ('squash_empty_commits' in
+                filter_processor.FilterProcessor.known_params):
+            params['squash_empty_commits'] = (not dont_squash_empty_commits)
+        else:
+            if dont_squash_empty_commits:
+                raise BzrCommandError("installed python-fastimport does not "
+                    "support not squashing empty commits. Please install "
+                    " a newer python-fastimport to use "
+                    "--dont-squash-empty-commits")
+
         from fastimport import parser
         stream = _get_source_stream(source)
         user_mapper = _get_user_mapper(user_map)
@@ -542,9 +556,9 @@ class cmd_fast_export(Command):
     :Round-tripping:
 
      Recent versions of the fast-import specification support features
-     that allow effective round-tripping of many Bazaar branches. As
-     such, fast-exporting a branch and fast-importing the data produced
-     will create a new repository with equivalent history, i.e.
+     that allow effective round-tripping most of the metadata in Bazaar
+     branches. As such, fast-exporting a branch and fast-importing the data
+     produced will create a new repository with roughly equivalent history, i.e.
      "bzr log -v -p --include-merges --forward" on the old branch and
      new branch should produce similar, if not identical, results.
 
@@ -579,6 +593,19 @@ class cmd_fast_export(Command):
      future once the feature names and definitions are formally agreed
      to by the broader fast-import developer community.
 
+     Git has stricter naming rules for tags and fast-export --plain
+     will skip tags which can't be imported into git. To replace characters
+     unsupported in git with an underscore instead, specify
+     --rewrite-tag-names.
+
+    :History truncation:
+
+     It is sometimes convenient to simply truncate the revision history at a
+     certain point.  The --baseline option, to be used in conjunction with -r,
+     emits a baseline commit containing the state of the entire source tree at
+     the first requested revision.  This allows a user to produce a tree
+     identical to the original without munging multiple exports.
+
     :Examples:
 
      To produce data destined for import into Bazaar::
@@ -625,22 +652,35 @@ class cmd_fast_export(Command):
                     Option('plain',
                         help="Exclude metadata to maximise interoperability."
                         ),
+                    Option('rewrite-tag-names',
+                        help="Replace characters invalid in git with '_'"
+                             " (plain mode only).",
+                        ),
+                    Option('baseline',
+                        help="Export an 'absolute' baseline commit prior to"
+                             "the first relative commit",
+                        ),
                      ]
     encoding_type = 'exact'
     def run(self, source, destination=None, verbose=False,
         git_branch="master", checkpoint=10000, marks=None,
         import_marks=None, export_marks=None, revision=None,
-        plain=True):
+        plain=True, rewrite_tag_names=False, baseline=False):
         load_fastimport()
+        from bzrlib.branch import Branch
         from bzrlib.plugins.fastimport import exporter
 
         if marks:
             import_marks = export_marks = marks
-        exporter = exporter.BzrFastExporter(source,
-            destination=destination,
-            git_branch=git_branch, checkpoint=checkpoint,
+
+        # Open the source
+        branch = Branch.open_containing(source)[0]
+        outf = exporter._get_output_stream(destination)
+        exporter = exporter.BzrFastExporter(branch,
+            outf=outf, git_branch=git_branch, checkpoint=checkpoint,
             import_marks_file=import_marks, export_marks_file=export_marks,
-            revision=revision, verbose=verbose, plain_format=plain)
+            revision=revision, verbose=verbose, plain_format=plain,
+            rewrite_tags=rewrite_tag_names, baseline=baseline)
         return exporter.run()
 
 
