@@ -13,11 +13,29 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-# Original Copyright (c) 2008 Adeodato Simó
-# Original License: MIT (See exporters/bzr-fast-export.LICENSE)
+# Based on bzr-fast-export
+# Copyright (c) 2008 Adeodato Simó
+#
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 # vim: fileencoding=utf-8
 
@@ -28,7 +46,7 @@
 # set new_git_branch to the previously used name)
 
 from email.Utils import parseaddr
-import sys, time
+import sys, time, re
 
 import bzrlib.branch
 import bzrlib.revision
@@ -94,21 +112,57 @@ def check_ref_format(refname):
     return True
 
 
+def sanitize_ref_name_for_git(refname):
+    """Rewrite refname so that it will be accepted by git-fast-import.
+    For the detailed rules see check_ref_format.
+
+    By rewriting the refname we are breaking uniqueness guarantees provided by bzr
+    so we have to manually
+    verify that resulting ref names are unique.
+
+    :param refname: refname to rewrite
+    :return: new refname
+    """
+    new_refname = re.sub(
+        # '/.' in refname or startswith '.'
+        r"/\.|^\."
+        # '..' in refname
+        r"|\.\."
+        # ord(c) < 040
+        r"|[" + "".join([chr(x) for x in range(040)]) + r"]"
+        # c in '\177 ~^:?*['
+        r"|[\177 ~^:?*[]"
+        # last char in "/."
+        r"|[/.]$"
+        # endswith '.lock'
+        r"|.lock$"
+        # "@{" in refname
+        r"|@{"
+        # "\\" in refname
+        r"|\\",
+        "_", refname)
+    return new_refname
+
 
 class BzrFastExporter(object):
 
-    def __init__(self, source, destination, git_branch=None, checkpoint=-1,
+    def __init__(self, source, outf, git_branch=None, checkpoint=-1,
         import_marks_file=None, export_marks_file=None, revision=None,
-        verbose=False, plain_format=False):
+        verbose=False, plain_format=False, rewrite_tags=False,
+        baseline=False):
         """Export branch data in fast import format.
 
         :param plain_format: if True, 'classic' fast-import format is
-          used without any extended features; if False, the generated
-          data is richer and includes information like multiple
-          authors, revision properties, etc.
+            used without any extended features; if False, the generated
+            data is richer and includes information like multiple
+            authors, revision properties, etc.
+        :param rewrite_tags: if True and if plain_format is set, tag names
+            will be rewritten to be git-compatible.
+            Otherwise tags which aren't valid for git will be skipped if
+            plain_format is set.
         """
-        self.source = source
-        self.outf = _get_output_stream(destination)
+        self.branch = source
+        self.outf = outf
         self.git_branch = git_branch
         self.checkpoint = checkpoint
         self.import_marks_file = import_marks_file
@@ -116,6 +170,8 @@ class BzrFastExporter(object):
         self.revision = revision
         self.excluded_revisions = set()
         self.plain_format = plain_format
+        self.rewrite_tags = rewrite_tags
+        self.baseline = baseline
         self._multi_author_api_available = hasattr(bzrlib.revision.Revision,
             'get_apparent_authors')
         self.properties_to_exclude = ['authors', 'author']
@@ -150,8 +206,9 @@ class BzrFastExporter(object):
             start_rev_id = None
             end_rev_id = None
         self.note("Calculating the revisions to include ...")
-        view_revisions = reversed([rev_id for rev_id, _, _, _ in
-            self.branch.iter_merge_sorted_revisions(end_rev_id, start_rev_id)])
+        view_revisions = [rev_id for rev_id, _, _, _ in
+            self.branch.iter_merge_sorted_revisions(end_rev_id, start_rev_id)]
+        view_revisions.reverse()
         # If a starting point was given, we need to later check that we don't
         # start emitting revisions from before that point. Collect the
         # revisions to exclude now ...
@@ -159,12 +216,13 @@ class BzrFastExporter(object):
             self.note("Calculating the revisions to exclude ...")
             self.excluded_revisions = set([rev_id for rev_id, _, _, _ in
                 self.branch.iter_merge_sorted_revisions(start_rev_id)])
+            if self.baseline:
+                # needed so the first relative commit knows its parent
+                self.excluded_revisions.remove(start_rev_id)
+                view_revisions.insert(0, start_rev_id)
         return list(view_revisions)
 
     def run(self):
-        # Open the source
-        self.branch = bzrlib.branch.Branch.open_containing(self.source)[0]
-
         # Export the data
         self.branch.repository.lock_read()
         try:
@@ -174,6 +232,8 @@ class BzrFastExporter(object):
                 self._commit_total)
             if not self.plain_format:
                 self.emit_features()
+            if self.baseline:
+                self.emit_baseline(interesting.pop(0), self.git_branch)
             for revid in interesting:
                 self.emit_commit(revid, self.git_branch)
             if self.branch.supports_tags():
@@ -251,6 +311,16 @@ class BzrFastExporter(object):
         for feature in sorted(commands.FEATURE_NAMES):
             self.print_cmd(commands.FeatureCommand(feature))
 
+    def emit_baseline(self, revid, git_branch):
+        # Emit a full source tree of the first commit's parent
+        git_ref = 'refs/heads/%s' % (git_branch,)
+        revobj = self.branch.repository.get_revision(revid)
+        mark = 1
+        self.revid_to_mark[revid] = mark
+        file_cmds = self._get_filecommands(bzrlib.revision.NULL_REVISION, revid)
+        self.print_cmd(self._get_commit_command(git_ref, mark, revobj,
+            file_cmds))
+
     def emit_commit(self, revid, git_branch):
         if revid in self.revid_to_mark or revid in self.excluded_revisions:
             return
@@ -541,9 +611,15 @@ class BzrFastExporter(object):
             else:
                 git_ref = 'refs/tags/%s' % tag.encode("utf-8")
                 if self.plain_format and not check_ref_format(git_ref):
-                    self.warning('not creating tag %r as its name would not be '
-                                 'valid in git.', git_ref)
-                    continue
+                    if self.rewrite_tags:
+                        new_ref = sanitize_ref_name_for_git(git_ref)
+                        self.warning('tag %r is exported as %r to be valid in git.',
+                                     git_ref, new_ref)
+                        git_ref = new_ref
+                    else:
+                        self.warning('not creating tag %r as its name would not be '
+                                     'valid in git.', git_ref)
+                        continue
                 self.print_cmd(commands.ResetCommand(git_ref, ":" + str(mark)))
 
     def _next_tmp_branch_name(self):
