@@ -11,37 +11,44 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Import processor that supports all Bazaar repository formats."""
 
 
 import time
 from bzrlib import (
-    bzrdir,
+    debug,
     delta,
     errors,
     osutils,
     progress,
     )
-from bzrlib.repofmt import pack_repo
-from bzrlib.trace import note, mutter
+try:
+    from bzrlib.repofmt.knitpack_repo import KnitPackRepository
+except ImportError:
+    from bzrlib.repofmt.pack_repo import KnitPackRepository
+from bzrlib.trace import (
+    mutter,
+    note,
+    warning,
+    )
 try:
     import bzrlib.util.configobj.configobj as configobj
 except ImportError:
     import configobj
 from bzrlib.plugins.fastimport import (
     branch_updater,
-    bzr_commit_handler,
     cache_manager,
+    marks_file,
+    revision_store,
+    )
+from fastimport import (
     commands,
     errors as plugin_errors,
     helpers,
     idmapfile,
-    marks_file,
     processor,
-    revision_store,
     )
 
 
@@ -122,8 +129,18 @@ class GenericProcessor(processor.ImportProcessor):
 
     def __init__(self, bzrdir, params=None, verbose=False, outf=None,
             prune_empty_dirs=True):
-        processor.ImportProcessor.__init__(self, bzrdir, params, verbose)
+        processor.ImportProcessor.__init__(self, params, verbose)
         self.prune_empty_dirs = prune_empty_dirs
+        self.bzrdir = bzrdir
+        try:
+            # Might be inside a branch
+            (self.working_tree, self.branch) = bzrdir._get_tree_branch()
+            self.repo = self.branch.repository
+        except errors.NotBranchError:
+            # Must be inside a repository
+            self.working_tree = None
+            self.branch = None
+            self.repo = bzrdir.open_repository()
 
     def pre_process(self):
         self._start_time = time.time()
@@ -135,11 +152,11 @@ class GenericProcessor(processor.ImportProcessor):
             self.note("Starting import ...")
         self.cache_mgr = cache_manager.CacheManager(self.info, self.verbose,
             self.inventory_cache_size)
-        
+
         if self.params.get("import-marks") is not None:
             mark_info = marks_file.import_marks(self.params.get("import-marks"))
             if mark_info is not None:
-                self.cache_mgr.revision_ids = mark_info[0]
+                self.cache_mgr.marks = mark_info
             self.skip_total = False
             self.first_incremental_commit = True
         else:
@@ -158,7 +175,7 @@ class GenericProcessor(processor.ImportProcessor):
 
         # Disable autopacking if the repo format supports it.
         # THIS IS A HACK - there is no sanctioned way of doing this yet.
-        if isinstance(self.repo, pack_repo.KnitPackRepository):
+        if isinstance(self.repo, KnitPackRepository):
             self._original_max_pack_count = \
                 self.repo._pack_collection._max_pack_count
             def _max_pack_count_for_import(total_revisions):
@@ -182,6 +199,7 @@ class GenericProcessor(processor.ImportProcessor):
         self.repo.start_write_group()
 
     def _load_info_and_params(self):
+        from bzrlib.plugins.fastimport import bzr_commit_handler
         self._mode = bool(self.params.get('mode', 'default'))
         self._experimental = self._mode == 'experimental'
 
@@ -277,6 +295,31 @@ class GenericProcessor(processor.ImportProcessor):
                 self.repo, self.inventory_cache_size,
                 fulltext_when=fulltext_when)
 
+    def process(self, command_iter):
+        """Import data into Bazaar by processing a stream of commands.
+
+        :param command_iter: an iterator providing commands
+        """
+        if self.working_tree is not None:
+            self.working_tree.lock_write()
+        elif self.branch is not None:
+            self.branch.lock_write()
+        elif self.repo is not None:
+            self.repo.lock_write()
+        try:
+            super(GenericProcessor, self)._process(command_iter)
+        finally:
+            # If an unhandled exception occurred, abort the write group
+            if self.repo is not None and self.repo.is_in_write_group():
+                self.repo.abort_write_group()
+            # Release the locks
+            if self.working_tree is not None:
+                self.working_tree.unlock()
+            elif self.branch is not None:
+                self.branch.unlock()
+            elif self.repo is not None:
+                self.repo.unlock()
+
     def _process(self, command_iter):
         # if anything goes wrong, abort the write group if any
         try:
@@ -293,17 +336,18 @@ class GenericProcessor(processor.ImportProcessor):
 
         if self.params.get("export-marks") is not None:
             marks_file.export_marks(self.params.get("export-marks"),
-                self.cache_mgr.revision_ids)
+                self.cache_mgr.marks)
 
-        if self.cache_mgr.last_ref == None:
+        if self.cache_mgr.reftracker.last_ref == None:
             """Nothing to refresh"""
             return
 
         # Update the branches
         self.note("Updating branch information ...")
         updater = branch_updater.BranchUpdater(self.repo, self.branch,
-            self.cache_mgr, helpers.invert_dictset(self.cache_mgr.heads),
-            self.cache_mgr.last_ref, self.tags)
+            self.cache_mgr, helpers.invert_dictset(
+                self.cache_mgr.reftracker.heads),
+            self.cache_mgr.reftracker.last_ref, self.tags)
         branches_updated, branches_lost = updater.update()
         self._branch_count = len(branches_updated)
 
@@ -432,7 +476,7 @@ class GenericProcessor(processor.ImportProcessor):
         # Currently, we just check the size. In the future, we might
         # decide to be more paranoid and check that the revision-ids
         # are identical as well.
-        self.cache_mgr.revision_ids, known = idmapfile.load_id_map(
+        self.cache_mgr.marks, known = idmapfile.load_id_map(
             self.id_map_path)
         existing_count = len(self.repo.all_revision_ids())
         if existing_count < known:
@@ -443,7 +487,7 @@ class GenericProcessor(processor.ImportProcessor):
         """Save the id-map."""
         # Save the whole lot every time. If this proves a problem, we can
         # change to 'append just the new ones' at a later time.
-        idmapfile.save_id_map(self.id_map_path, self.cache_mgr.revision_ids)
+        idmapfile.save_id_map(self.id_map_path, self.cache_mgr.marks)
 
     def blob_handler(self, cmd):
         """Process a BlobCommand."""
@@ -467,14 +511,12 @@ class GenericProcessor(processor.ImportProcessor):
 
     def commit_handler(self, cmd):
         """Process a CommitCommand."""
+        mark = cmd.id.lstrip(':')
         if self.skip_total and self._revision_count < self.skip_total:
-            self.cache_mgr.track_heads(cmd)
+            self.cache_mgr.reftracker.track_heads(cmd)
             # Check that we really do know about this commit-id
-            if not self.cache_mgr.revision_ids.has_key(cmd.id):
-                raise plugin_errors.BadRestart(cmd.id)
-            # Consume the file commands and free any non-sticky blobs
-            for fc in cmd.file_iter():
-                pass
+            if not self.cache_mgr.marks.has_key(mark):
+                raise plugin_errors.BadRestart(mark)
             self.cache_mgr._blobs = {}
             self._revision_count += 1
             if cmd.ref.startswith('refs/tags/'):
@@ -483,7 +525,7 @@ class GenericProcessor(processor.ImportProcessor):
             return
         if self.first_incremental_commit:
             self.first_incremental_commit = None
-            parents = self.cache_mgr.track_heads(cmd)
+            parents = self.cache_mgr.reftracker.track_heads(cmd)
 
         # 'Commit' the revision and report progress
         handler = self.commit_handler_factory(cmd, self.cache_mgr,
@@ -494,9 +536,9 @@ class GenericProcessor(processor.ImportProcessor):
         except:
             print "ABORT: exception occurred processing commit %s" % (cmd.id)
             raise
-        self.cache_mgr.revision_ids[cmd.id] = handler.revision_id
+        self.cache_mgr.add_mark(mark, handler.revision_id)
         self._revision_count += 1
-        self.report_progress("(%s)" % cmd.id)
+        self.report_progress("(%s)" % cmd.id.lstrip(':'))
 
         if cmd.ref.startswith('refs/tags/'):
             tag_name = cmd.ref[len('refs/tags/'):]
@@ -546,7 +588,7 @@ class GenericProcessor(processor.ImportProcessor):
             return
 
         if cmd.from_ is not None:
-            self.cache_mgr.track_heads_for_ref(cmd.ref, cmd.from_)
+            self.cache_mgr.reftracker.track_heads_for_ref(cmd.ref, cmd.from_)
 
     def tag_handler(self, cmd):
         """Process a TagCommand."""
@@ -558,7 +600,7 @@ class GenericProcessor(processor.ImportProcessor):
     def _set_tag(self, name, from_):
         """Define a tag given a name and import 'from' reference."""
         bzr_tag_name = name.decode('utf-8', 'replace')
-        bzr_rev_id = self.cache_mgr.revision_ids[from_]
+        bzr_rev_id = self.cache_mgr.lookup_committish(from_)
         self.tags[bzr_tag_name] = bzr_rev_id
 
     def feature_handler(self, cmd):
@@ -566,3 +608,19 @@ class GenericProcessor(processor.ImportProcessor):
         feature = cmd.feature_name
         if feature not in commands.FEATURE_NAMES:
             raise plugin_errors.UnknownFeature(feature)
+
+    def debug(self, msg, *args):
+        """Output a debug message if the appropriate -D option was given."""
+        if "fast-import" in debug.debug_flags:
+            msg = "%s DEBUG: %s" % (self._time_of_day(), msg)
+            mutter(msg, *args)
+
+    def note(self, msg, *args):
+        """Output a note but timestamp it."""
+        msg = "%s %s" % (self._time_of_day(), msg)
+        note(msg, *args)
+
+    def warning(self, msg, *args):
+        """Output a warning but timestamp it."""
+        msg = "%s WARNING: %s" % (self._time_of_day(), msg)
+        warning(msg, *args)
