@@ -11,24 +11,24 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import time
 
 from bzrlib import (
-    branch,
     tests,
     )
-
-from bzrlib.plugins.fastimport import (
-    commands,
-    errors,
+from bzrlib.plugins.fastimport.helpers import (
+    kind_to_mode,
+    )
+from bzrlib.plugins.fastimport.tests import (
+    FastimportFeature,
     )
 
-from bzrlib.plugins.fastimport.processors import (
-    generic_processor,
-    )
+try:
+    from fastimport import commands
+except ImportError:
+    commands = object()
 
 
 def load_tests(standard_tests, module, loader):
@@ -49,9 +49,14 @@ def load_tests(standard_tests, module, loader):
 
 class TestCaseForGenericProcessor(tests.TestCaseWithTransport):
 
+    _test_needs_features = [FastimportFeature]
+
     branch_format = "pack-0.92"
 
     def get_handler(self):
+        from bzrlib.plugins.fastimport.processors import (
+            generic_processor,
+            )
         branch = self.make_branch('.', format=self.branch_format)
         handler = generic_processor.GenericProcessor(branch.bzrdir)
         return handler, branch
@@ -188,7 +193,7 @@ class TestCaseForGenericProcessor(tests.TestCaseWithTransport):
                          revtree.inventory.root.children[path].revision)
 
 
-class TestImportToPackModify(TestCaseForGenericProcessor):
+class TestImportToPackTag(TestCaseForGenericProcessor):
 
     def file_command_iter(self, path, kind='file', content='aaa',
         executable=False, to_kind=None, to_content='bbb', to_executable=None):
@@ -202,13 +207,54 @@ class TestImportToPackModify(TestCaseForGenericProcessor):
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(path, kind, executable,
-                        None, content)
+                yield commands.FileModifyCommand(path,
+                    kind_to_mode(kind, executable), None, content)
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
             def files_two():
-                yield commands.FileModifyCommand(path, to_kind, to_executable,
-                        None, to_content)
+                yield commands.FileModifyCommand(path,
+                    kind_to_mode(to_kind, to_executable), None, to_content)
+
+            # pass "head" for from_ to show that #401249 is worse than I knew
+            yield commands.CommitCommand('head', '2', author,
+                committer, "commit 2", "head", [], files_two)
+
+            yield commands.TagCommand('tag1', ':1', committer, "tag 1")
+
+            # pass "head" for from_ to demonstrate #401249
+            yield commands.TagCommand('tag2', 'head', committer, "tag 2")
+        return command_list
+
+    def test_tag(self):
+        handler, branch = self.get_handler()
+        path = 'a'
+        raise tests.KnownFailure("non-mark committish not yet supported"
+                                 "- bug #410249")
+        handler.process(self.file_command_iter(path))
+
+
+class TestImportToPackModify(TestCaseForGenericProcessor):
+
+    def file_command_iter(self, path, kind='file', content='aaa',
+        executable=False, to_kind=None, to_content='bbb', to_executable=None):
+
+        # Revno 1: create a file or symlink
+        # Revno 2: modify it
+        if to_kind is None:
+            to_kind = kind
+        if to_executable is None:
+            to_executable = executable
+        mode = kind_to_mode(kind, executable)
+        to_mode = kind_to_mode(to_kind, to_executable)
+        def command_list():
+            author = ['', 'bugs@a.com', time.time(), time.timezone]
+            committer = ['', 'elmer@a.com', time.time(), time.timezone]
+            def files_one():
+                yield commands.FileModifyCommand(path, mode, None, content)
+            yield commands.CommitCommand('head', '1', author,
+                committer, "commit 1", None, [], files_one)
+            def files_two():
+                yield commands.FileModifyCommand(path, to_mode, None, to_content)
             yield commands.CommitCommand('head', '2', author,
                 committer, "commit 2", ":1", [], files_two)
         return command_list
@@ -310,12 +356,13 @@ class TestImportToPackModify(TestCaseForGenericProcessor):
 
 class TestImportToPackModifyTwice(TestCaseForGenericProcessor):
     """This tests when the same file is modified twice in the one commit.
-    
+
     Note: hg-fast-export produces data like this on occasions.
     """
 
     def file_command_iter(self, path, kind='file', content='aaa',
         executable=False, to_kind=None, to_content='bbb', to_executable=None):
+
         # Revno 1: create a file twice
         if to_kind is None:
             to_kind = kind
@@ -325,9 +372,9 @@ class TestImportToPackModifyTwice(TestCaseForGenericProcessor):
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(path, kind, executable,
+                yield commands.FileModifyCommand(path, kind_to_mode(kind, executable),
                         None, content)
-                yield commands.FileModifyCommand(path, to_kind, to_executable,
+                yield commands.FileModifyCommand(path, kind_to_mode(to_kind, to_executable),
                         None, to_content)
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
@@ -346,6 +393,7 @@ class TestImportToPackModifyTwice(TestCaseForGenericProcessor):
 class TestImportToPackModifyTricky(TestCaseForGenericProcessor):
 
     def file_command_iter(self, path1, path2, kind='file'):
+
         # Revno 1: create a file or symlink in a directory
         # Revno 2: create a second file that implicitly deletes the
         # first one because either:
@@ -355,12 +403,12 @@ class TestImportToPackModifyTricky(TestCaseForGenericProcessor):
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(path1, kind, False,
+                yield commands.FileModifyCommand(path1, kind_to_mode(kind, False),
                         None, "aaa")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
             def files_two():
-                yield commands.FileModifyCommand(path2, kind, False,
+                yield commands.FileModifyCommand(path2, kind_to_mode(kind, False),
                         None, "bbb")
             yield commands.CommitCommand('head', '2', author,
                 committer, "commit 2", ":1", [], files_two)
@@ -423,13 +471,14 @@ class TestImportToPackModifyTricky(TestCaseForGenericProcessor):
 class TestImportToPackDelete(TestCaseForGenericProcessor):
 
     def file_command_iter(self, path, kind='file'):
+
         # Revno 1: create a file or symlink
         # Revno 2: delete it
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(path, kind, False,
+                yield commands.FileModifyCommand(path, kind_to_mode(kind, False),
                         None, "aaa")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
@@ -494,12 +543,13 @@ class TestImportToPackDeleteNew(TestCaseForGenericProcessor):
     """Test deletion of a newly added file."""
 
     def file_command_iter(self, path, kind='file'):
+
         # Revno 1: create a file or symlink then delete it
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(path, kind, False,
+                yield commands.FileModifyCommand(path, kind_to_mode(kind, False),
                         None, "aaa")
                 yield commands.FileDeleteCommand(path)
             yield commands.CommitCommand('head', '1', author,
@@ -540,6 +590,7 @@ class TestImportToPackDeleteNew(TestCaseForGenericProcessor):
 class TestImportToPackDeleteMultiLevel(TestCaseForGenericProcessor):
 
     def file_command_iter(self, paths, paths_to_delete):
+
         # Revno 1: create multiple files
         # Revno 2: delete multiple files
         def command_list():
@@ -547,7 +598,7 @@ class TestImportToPackDeleteMultiLevel(TestCaseForGenericProcessor):
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
                 for i, path in enumerate(paths):
-                    yield commands.FileModifyCommand(path, 'file', False,
+                    yield commands.FileModifyCommand(path, kind_to_mode('file', False),
                             None, "aaa%d" % i)
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
@@ -615,6 +666,7 @@ class TestImportToPackDeleteThenAdd(TestCaseForGenericProcessor):
 
     def file_command_iter(self, path, kind='file', content='aaa',
         executable=False, to_kind=None, to_content='bbb', to_executable=None):
+
         # Revno 1: create a file or symlink
         # Revno 2: delete it and add it
         if to_kind is None:
@@ -625,13 +677,13 @@ class TestImportToPackDeleteThenAdd(TestCaseForGenericProcessor):
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(path, kind, executable,
+                yield commands.FileModifyCommand(path, kind_to_mode(kind, executable),
                         None, content)
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
             def files_two():
                 yield commands.FileDeleteCommand(path)
-                yield commands.FileModifyCommand(path, to_kind, to_executable,
+                yield commands.FileModifyCommand(path, kind_to_mode(to_kind, to_executable),
                         None, to_content)
             yield commands.CommitCommand('head', '2', author,
                 committer, "commit 2", ":1", [], files_two)
@@ -691,6 +743,7 @@ class TestImportToPackDeleteThenAdd(TestCaseForGenericProcessor):
 class TestImportToPackDeleteDirectory(TestCaseForGenericProcessor):
 
     def file_command_iter(self, paths, dir):
+
         # Revno 1: create multiple files
         # Revno 2: delete a directory holding those files
         def command_list():
@@ -698,7 +751,7 @@ class TestImportToPackDeleteDirectory(TestCaseForGenericProcessor):
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
                 for i, path in enumerate(paths):
-                    yield commands.FileModifyCommand(path, 'file', False,
+                    yield commands.FileModifyCommand(path, kind_to_mode('file', False),
                             None, "aaa%d" % i)
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
@@ -732,6 +785,7 @@ class TestImportToPackDeleteDirectoryThenAddFile(TestCaseForGenericProcessor):
     """Test deleting a directory then adding a file in the same commit."""
 
     def file_command_iter(self, paths, dir, new_path, kind='file'):
+
         # Revno 1: create files in a directory
         # Revno 2: delete the directory then add a file into it
         def command_list():
@@ -739,13 +793,13 @@ class TestImportToPackDeleteDirectoryThenAddFile(TestCaseForGenericProcessor):
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
                 for i, path in enumerate(paths):
-                    yield commands.FileModifyCommand(path, kind, False,
+                    yield commands.FileModifyCommand(path, kind_to_mode(kind, False),
                             None, "aaa%d" % i)
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
             def files_two():
                 yield commands.FileDeleteCommand(dir)
-                yield commands.FileModifyCommand(new_path, kind, False,
+                yield commands.FileModifyCommand(new_path, kind_to_mode(kind, False),
                         None, "bbb")
             yield commands.CommitCommand('head', '2', author,
                 committer, "commit 2", ":1", [], files_two)
@@ -781,13 +835,14 @@ class TestImportToPackDeleteDirectoryThenAddFile(TestCaseForGenericProcessor):
 class TestImportToPackRename(TestCaseForGenericProcessor):
 
     def get_command_iter(self, old_path, new_path, kind='file'):
+
         # Revno 1: create a file or symlink
         # Revno 2: rename it
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(old_path, kind, False,
+                yield commands.FileModifyCommand(old_path, kind_to_mode(kind, False),
                         None, "aaa")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
@@ -856,12 +911,13 @@ class TestImportToPackRenameNew(TestCaseForGenericProcessor):
     """Test rename of a newly added file."""
 
     def get_command_iter(self, old_path, new_path, kind='file'):
+
         # Revno 1: create a file and rename it
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(old_path, kind, False,
+                yield commands.FileModifyCommand(old_path, kind_to_mode(kind, False),
                         None, "aaa")
                 yield commands.FileRenameCommand(old_path, new_path)
             yield commands.CommitCommand('head', '1', author,
@@ -907,15 +963,16 @@ class TestImportToPackRenameToDeleted(TestCaseForGenericProcessor):
     """Test rename to a destination path deleted in this commit."""
 
     def get_command_iter(self, old_path, new_path, kind='file'):
+
         # Revno 1: create two files
         # Revno 2: delete one, rename the other one to that path
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(old_path, kind, False,
+                yield commands.FileModifyCommand(old_path, kind_to_mode(kind, False),
                         None, "aaa")
-                yield commands.FileModifyCommand(new_path, kind, False,
+                yield commands.FileModifyCommand(new_path, kind_to_mode(kind, False),
                         None, "bbb")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
@@ -1019,18 +1076,19 @@ class TestImportToPackRenameModified(TestCaseForGenericProcessor):
     """Test rename of a path previously modified in this commit."""
 
     def get_command_iter(self, old_path, new_path, kind='file'):
+
         # Revno 1: create a file or symlink
         # Revno 2: modify then rename it
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(old_path, kind, False,
+                yield commands.FileModifyCommand(old_path, kind_to_mode(kind, False),
                         None, "aaa")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
             def files_two():
-                yield commands.FileModifyCommand(old_path, kind, False,
+                yield commands.FileModifyCommand(old_path, kind_to_mode(kind, False),
                         None, "bbb")
                 yield commands.FileRenameCommand(old_path, new_path)
             yield commands.CommitCommand('head', '2', author,
@@ -1134,19 +1192,20 @@ class TestImportToPackRenameThenModify(TestCaseForGenericProcessor):
     """Test rename of a path then modfy the new-path in the same commit."""
 
     def get_command_iter(self, old_path, new_path, kind='file'):
+
         # Revno 1: create a file or symlink
         # Revno 2: rename it then modify the newly created path
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(old_path, kind, False,
+                yield commands.FileModifyCommand(old_path, kind_to_mode(kind, False),
                         None, "aaa")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
             def files_two():
                 yield commands.FileRenameCommand(old_path, new_path)
-                yield commands.FileModifyCommand(new_path, kind, False,
+                yield commands.FileModifyCommand(new_path, kind_to_mode(kind, False),
                         None, "bbb")
             yield commands.CommitCommand('head', '2', author,
                 committer, "commit 2", ":1", [], files_two)
@@ -1249,22 +1308,23 @@ class TestImportToPackDeleteRenameThenModify(TestCaseForGenericProcessor):
     """Test rename of to a deleted path then modfy the new-path in the same commit."""
 
     def get_command_iter(self, old_path, new_path, kind='file'):
+
         # Revno 1: create two files or symlinks
         # Revno 2: delete one, rename the other to it then modify the newly created path
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(old_path, kind, False,
+                yield commands.FileModifyCommand(old_path, kind_to_mode(kind, False),
                         None, "aaa")
-                yield commands.FileModifyCommand(new_path, kind, False,
+                yield commands.FileModifyCommand(new_path, kind_to_mode(kind, False),
                         None, "zzz")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
             def files_two():
                 yield commands.FileDeleteCommand(new_path)
                 yield commands.FileRenameCommand(old_path, new_path)
-                yield commands.FileModifyCommand(new_path, kind, False,
+                yield commands.FileModifyCommand(new_path, kind_to_mode(kind, False),
                         None, "bbb")
             yield commands.CommitCommand('head', '2', author,
                 committer, "commit 2", ":1", [], files_two)
@@ -1376,6 +1436,7 @@ class TestImportToPackDeleteRenameThenModify(TestCaseForGenericProcessor):
 class TestImportToPackRenameTricky(TestCaseForGenericProcessor):
 
     def file_command_iter(self, path1, old_path2, new_path2, kind='file'):
+
         # Revno 1: create two files or symlinks in a directory
         # Revno 2: rename the second file so that it implicitly deletes the
         # first one because either:
@@ -1385,9 +1446,9 @@ class TestImportToPackRenameTricky(TestCaseForGenericProcessor):
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(path1, kind, False,
+                yield commands.FileModifyCommand(path1, kind_to_mode(kind, False),
                         None, "aaa")
-                yield commands.FileModifyCommand(old_path2, kind, False,
+                yield commands.FileModifyCommand(old_path2, kind_to_mode(kind, False),
                         None, "bbb")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
@@ -1459,13 +1520,14 @@ class TestImportToPackRenameTricky(TestCaseForGenericProcessor):
 class TestImportToPackCopy(TestCaseForGenericProcessor):
 
     def file_command_iter(self, src_path, dest_path, kind='file'):
+
         # Revno 1: create a file or symlink
         # Revno 2: copy it
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(src_path, kind, False,
+                yield commands.FileModifyCommand(src_path, kind_to_mode(kind, False),
                         None, "aaa")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
@@ -1550,12 +1612,13 @@ class TestImportToPackCopyNew(TestCaseForGenericProcessor):
     """Test copy of a newly added file."""
 
     def file_command_iter(self, src_path, dest_path, kind='file'):
+
         # Revno 1: create a file or symlink and copy it
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(src_path, kind, False,
+                yield commands.FileModifyCommand(src_path, kind_to_mode(kind, False),
                         None, "aaa")
                 yield commands.FileCopyCommand(src_path, dest_path)
             yield commands.CommitCommand('head', '1', author,
@@ -1630,15 +1693,16 @@ class TestImportToPackCopyNew(TestCaseForGenericProcessor):
 class TestImportToPackCopyToDeleted(TestCaseForGenericProcessor):
 
     def file_command_iter(self, src_path, dest_path, kind='file'):
+
         # Revno 1: create two files or symlinks
         # Revno 2: delete one and copy the other one to its path
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(src_path, kind, False,
+                yield commands.FileModifyCommand(src_path, kind_to_mode(kind, False),
                         None, "aaa")
-                yield commands.FileModifyCommand(dest_path, kind, False,
+                yield commands.FileModifyCommand(dest_path, kind_to_mode(kind, False),
                         None, "bbb")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
@@ -1718,18 +1782,19 @@ class TestImportToPackCopyModified(TestCaseForGenericProcessor):
     """Test copy of file/symlink already modified in this commit."""
 
     def file_command_iter(self, src_path, dest_path, kind='file'):
+
         # Revno 1: create a file or symlink
         # Revno 2: modify and copy it
         def command_list():
             author = ['', 'bugs@a.com', time.time(), time.timezone]
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(src_path, kind, False,
+                yield commands.FileModifyCommand(src_path, kind_to_mode(kind, False),
                         None, "aaa")
             yield commands.CommitCommand('head', '1', author,
                 committer, "commit 1", None, [], files_one)
             def files_two():
-                yield commands.FileModifyCommand(src_path, kind, False,
+                yield commands.FileModifyCommand(src_path, kind_to_mode(kind, False),
                         None, "bbb")
                 yield commands.FileCopyCommand(src_path, dest_path)
             yield commands.CommitCommand('head', '2', author,
@@ -1816,10 +1881,11 @@ class TestImportToPackCopyModified(TestCaseForGenericProcessor):
 class TestImportToPackFileKinds(TestCaseForGenericProcessor):
 
     def get_command_iter(self, path, kind, content):
+
         def command_list():
             committer = ['', 'elmer@a.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand(path, kind, False,
+                yield commands.FileModifyCommand(path, kind_to_mode(kind, False),
                         None, content)
             yield commands.CommitCommand('head', '1', None,
                 committer, "commit 1", None, [], files_one)
@@ -1850,17 +1916,17 @@ class TestModifyRevertInBranch(TestCaseForGenericProcessor):
             committer_c = ['', 'c@elmer.com', time.time(), time.timezone]
             committer_d = ['', 'd@elmer.com', time.time(), time.timezone]
             def files_one():
-                yield commands.FileModifyCommand('foo', 'file', False,
+                yield commands.FileModifyCommand('foo', kind_to_mode('file', False),
                         None, "content A\n")
             yield commands.CommitCommand('head', '1', None,
                 committer_a, "commit 1", None, [], files_one)
             def files_two():
-                yield commands.FileModifyCommand('foo', 'file', False,
+                yield commands.FileModifyCommand('foo', kind_to_mode('file', False),
                         None, "content B\n")
             yield commands.CommitCommand('head', '2', None,
                 committer_b, "commit 2", ":1", [], files_two)
             def files_three():
-                yield commands.FileModifyCommand('foo', 'file', False,
+                yield commands.FileModifyCommand('foo', kind_to_mode('file', False),
                         None, "content A\n")
             yield commands.CommitCommand('head', '3', None,
                 committer_c, "commit 3", ":2", [], files_three)
@@ -1883,3 +1949,44 @@ class TestModifyRevertInBranch(TestCaseForGenericProcessor):
         self.assertEqual(rev_b, rtree_b.inventory[foo_id].revision)
         self.assertEqual(rev_c, rtree_c.inventory[foo_id].revision)
         self.assertEqual(rev_c, rtree_d.inventory[foo_id].revision)
+
+
+class TestCommitCommands(TestCaseForGenericProcessor):
+
+    def test_non_utf8_commit_message(self):
+        handler, branch = self.get_handler()
+        def files_one():
+            yield commands.FileModifyCommand('a',
+                kind_to_mode('file', False), None, "data")
+        def command_list():
+            committer = ['', 'elmer@a.com', time.time(), time.timezone]
+            yield commands.CommitCommand('head', '1', None,
+                committer, 'This is a funky character: \x83', None, [],
+                files_one)
+        handler.process(command_list)
+        rev = branch.repository.get_revision(branch.last_revision())
+        self.assertEquals(u"This is a funky character: \ufffd", rev.message)
+
+
+class TestAddNonUtf8InBranch(TestCaseForGenericProcessor):
+
+    def file_command_iter(self):
+        # A     add 'foo\x83'
+        def command_list():
+            committer_a = ['', 'a@elmer.com', time.time(), time.timezone]
+            def files_one():
+                yield commands.FileModifyCommand(
+                    'foo\x83', kind_to_mode('file', False), None, "content A\n")
+            yield commands.CommitCommand('head', '1', None,
+                committer_a, "commit 1", None, [], files_one)
+        return command_list
+
+    def test_add(self):
+        handler, branch = self.get_handler()
+        handler.process(self.file_command_iter())
+        branch.lock_read()
+        self.addCleanup(branch.unlock)
+        rev_a = branch.last_revision()
+        rtree_a = branch.repository.revision_tree(rev_a)
+        foo_id = rtree_a.path2id(u'foo\ufffd')
+        self.assertEqual(rev_a, rtree_a.inventory[foo_id].revision)
